---
- name: default
  description: Puppet BIND Pool
  attributes: {}

  ns_records:
    - hostname: ns1.example.com.
      priority: 1

  nameservers:
    - host: 127.0.0.1
      port: 5322

  targets:
    - type: bind9
      description: BIND Instance

      masters:
        # mdns doesn't support ipv6
        - host: 127.0.0.1
          port: 5354

      options:
        host: 127.0.0.1
        port: 5322
        rndc_host: 127.0.0.1
        rndc_port: 953
        rndc_config_file: /etc/bind/rndc.conf
        rndc_key_file: /etc/bind/rndc.key
