directory "/var/cache/bind";

recursion no;
allow-query { any; };
dnssec-enable yes;
dnssec-validation yes;

empty-zones-enable yes;

notify yes;
listen-on-v6 { any; };

minimal-responses yes;

allow-new-zones yes;
listen-on port 53 { any; };
listen-on-v6 port 53 { any; };
